/*
 *      Program do przegladania logow z ekg.
 *      Klasa obslugi wejscia z klawiatury.
 *
 *
 *      copyright (c) 2003 by Lukasz Bartnik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "input.hh"


input::input(int startx, int starty, int width):_act_line(0) {
    _win = newwin(1, width, starty, startx);
    if(!_win)
	error_exit("Nie mona otworzy okna ncurses dla okna input.");

    keypad(_win, TRUE);
}


input::~input(void) {
    delwin(_win);

    for(std::vector<command_base*>::iterator i=_commands.begin();
	i!=_commands.end(); ++i)
    {
	delete *i;
    }
}


/*
 * input::add_key()
 *
 * pobiera kod klawisza i wykonuje odpowiednia operacje
 *
 *  - key - wcisniety klawisz
 */
void input::add_key(int key) {
    switch(key) {
    case 0x0A:  /* ENTER */
	{
	    if(!_line.size())
		if(_history.size()) _line = _history.back();
		else                return;
	    else
		add_to_history(_line);

	    parse_command();
	    _line.erase();
	    try {
		if(_argv.size())
		    execute_command();
	    }
	    catch(...) {
		_argv.clear();
		throw;
	    }
	    _argv.clear();
	}
	break;
    case 0x107: /* BACKSPACE */
	if(_line.size())
	    _line.erase(_line.size()-1);
	break;
    default:
	_line += (char)key;
	break;
    }
}


/*
 * input::refresh()
 *
 * wypisz na ekran aktualna zawartosc paska
 */
void input::refresh(void) {
    werase(_win);
    wmove(_win, 0, 0);
    wprintl(_win, _line);

    wnoutrefresh(_win);
}


/*
 * input::parse_command()
 *
 * analizuje komende i alokuje pamiec na wszystkie argumenty
 */
void input::parse_command(void) {
    using std::string;
    string tmp;
    bool spc=false;

    string::size_type i=0;
    while(i < _line.size()) {
	switch(_line[i]) {
	    case '\\':
		if(_line[++i] == ' ')
		    tmp += ' ';
		else {
		    tmp += '\\';
		    tmp += _line[++i];
		}

		break;

	    case '\"':
		spc = !spc;

		break;

	    case ' ':
		if(spc)
		    tmp += ' ';
		else {
		    _argv.push_back(tmp);
		    tmp.erase();
		}

		break;
	    
	    default:
		tmp += _line[i];
	}
	++i;
    }
    if(tmp.size())
	_argv.push_back(tmp);

    if(!_argv.size())
	_argv.push_back(_line);
}


/*
 * input::execute_command()
 *
 * wykonuje komende wg. sparsowanej linii polecen
 */
void input::execute_command(void) {
    using std::string;

    string command_id = _argv[0];
    _argv.erase(_argv.begin());

    std::vector<command_base*>::iterator i;
    for(i=_commands.begin(); i!=_commands.end(); ++i) {
	if((**i) == command_id) {
	    (*i)->run(_argv);
	    return;
	}
    }
    if(i == _commands.end()) {
	_argv.clear();
	throw bad_command(command_id);
    }
}


/*
 * input::register_command()
 *
 * rejestruje nowa komende
 * przy zabijaniu okna na kazdym wskazniku wykonywane jest delete
 *
 *  - c - wskaznik na obiekt komendy
 */
void input::register_command(command_base *c) {
    _commands.push_back(c);
}


/*
 * funkcje do obslugi historii
 */
void input::up(void) {
    if(!_history.size())
	return;

    if(_act_line>0)
	--_act_line;

    _line = _history[_act_line];
}

void input::down(void) {
    if(!_history.size() || _act_line == _history.size())
	return;

    ++_act_line;
    if(_act_line == _history.size())
	_line.erase();
    else
	_line = _history[_act_line];
    
}

void input::pgup(void) {
}

void input::pgdown(void) {
}
